@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: BSD-2-Clause
  *
- * Copyright (c) 2018-2020 Gavin D. Howard and contributors.
+ * Copyright (c) 2018-2021 Gavin D. Howard and contributors.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -37,10 +37,19 @@
 #define BC_ARGS_H
 
 #include <status.h>
+#include <opt.h>
 #include <vm.h>
 
-void bc_args(int argc, char *argv[]);
+/**
+ * Processes command-line arguments.
+ * @param argc        How many arguments there are.
+ * @param argv        The array of arguments.
+ * @param exit_exprs  True if bc/dc should exit when there are expressions,
+ *                    false otherwise.
+ */
+void bc_args(int argc, char *argv[], bool exit_exprs);
 
-extern const char* const bc_args_env_name;
+// A reference to the list of long options.
+extern const BcOptLong bc_args_lopt[];
 
 #endif // BC_ARGS_H
