@@ -176,6 +176,9 @@
 /// The flag for reset on SIGINT.
 #define BC_FLAG_SIGINT (UINTMAX_C(1)<<12)
 
+/// The flag for exiting with expressions.
+#define BC_FLAG_EXPR_EXIT (UINTMAX_C(1)<<13)
+
 /// A convenience macro for getting the TTYIN flag.
 #define BC_TTYIN (vm.flags & BC_FLAG_TTYIN)
 
@@ -220,6 +223,9 @@
 /// A convenience macro for getting the leading zero flag.
 #define BC_Z (vm.flags & BC_FLAG_Z)
 
+/// A convenience macro for getting the expression exit flag.
+#define BC_EXPR_EXIT (vm.flags & BC_FLAG_EXPR_EXIT)
+
 #if BC_ENABLED
 
 /// A convenience macro for checking if bc is in POSIX mode.
@@ -539,8 +545,10 @@ typedef struct BcVm {
 	/// The messages for each error.
 	const char *err_msgs[BC_ERR_NELEMS];
 
+#if BC_ENABLE_NLS
 	/// The locale.
 	const char *locale;
+#endif // BC_ENABLE_NLS
 
 #endif // !BC_ENABLE_LIBRARY
 
