@@ -202,7 +202,7 @@ const uchar bc_err_ids[] = {
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
-	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
+	BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE, BC_ERR_IDX_PARSE,
 #endif // BC_ENABLED
 
 };
@@ -273,6 +273,7 @@ const char* const bc_err_msgs[] = {
 	"POSIX does not allow comparison operators outside if statements or loops",
 	"POSIX requires 0 or 1 comparison operators per condition",
 	"POSIX requires all 3 parts of a for loop to be non-empty",
+	"POSIX requires a newline between a semicolon and a function definition",
 #if BC_ENABLE_EXTRA_MATH
 	"POSIX does not allow exponential notation",
 #else
