@@ -27,15 +27,27 @@
 # POSSIBILITY OF SUCH DAMAGE.
 #
 
-set -e
-
 script="$0"
 
 testdir=$(dirname "${script}")
 
+pids=""
+
+# We need to figure out if we should run stuff in parallel.
+pll=1
+
+while getopts "n" opt; do
+
+	case "$opt" in
+		n) pll=0 ; shift ; set -e ;;
+		?) usage "Invalid option: $opt" ;;
+	esac
+
+done
+
 # Command-line processing.
 if [ "$#" -eq 0 ]; then
-	printf 'usage: %s dir [run_extra_tests] [run_stack_tests] [generate_tests] [time_tests] [exec args...]\n' "$script"
+	printf 'usage: %s [-n] dir [run_extra_tests] [run_stack_tests] [generate_tests] [time_tests] [exec args...]\n' "$script"
 	exit 1
 else
 	d="$1"
@@ -85,7 +97,36 @@ scripts=$(cat "$scriptdir/all.txt")
 for s in $scripts; do
 
 	f=$(basename "$s")
-	sh "$testdir/script.sh" "$d" "$f" "$run_extra_tests" "$run_stack_tests" \
-		"$generate" "$time_tests" "$exe" "$@"
+
+	if [ "$pll" -ne 0 ]; then
+		sh "$testdir/script.sh" "$d" "$f" "$run_extra_tests" "$run_stack_tests" \
+			"$generate" "$time_tests" "$exe" "$@" &
+		pids="$pids $!"
+	else
+		sh "$testdir/script.sh" "$d" "$f" "$run_extra_tests" "$run_stack_tests" \
+			"$generate" "$time_tests" "$exe" "$@"
+	fi
 
 done
+
+if [ "$pll" -ne 0 ]; then
+
+	exit_err=0
+
+	for p in $pids; do
+
+		wait "$p"
+		err="$?"
+
+		if [ "$err" -ne 0 ]; then
+			printf 'A script failed!\n'
+			exit_err=1
+		fi
+
+	done
+
+	if [ "$exit_err" -ne 0 ]; then
+		exit 1
+	fi
+
+fi
